/*
 * Proiect PM - 2016 - Belmega Diana-Elena, 332CA
 * harpa_laser.c
 */

#define F_CPU 16000000

#include <avr/io.h>
#include <util/delay.h>

#define OCTAVA1			1
#define OCTAVA2			2
#define OCTAVA3			3

#define NUM_NOTE		8
#define DURATA_NOTA		25
#define DURATA_NOTA_OCTAVA3 40

#define AREF 5
#define THRESHOLD 0.5

int octava = 1;

float frecvente_octava1[8] = {
	523.25, 587.33, 659.25, 698.46, 783.99, 880.0, 987.77, 1046.50
};

float frecvente_octava2[8] = {
	1046.50, 1174.66, 1318.51, 1396.91, 1567.98, 1760.00, 1975.53, 2093.00
};

float frecvente_octava3[8] = {
	2093.00, 2349.32, 2637.02, 2793.83, 3135.96, 3520.00, 3951.07, 4186.01
};


void IO_init(void) {
	/* Setez pinii PB0, PB1 si PB2 ca intrare. */
	DDRB &= ~(1 << PB0) & ~(1 << PB1) & ~(1 << PB2);
	
	/* Activez rezistenta de pull-up pentru pinii PB0, PB1 si PB2. */
	PORTB |= (1 << PB0) | (1 << PB1) | (1 << PB2);
	
	/* Setez pinii PB3, PB4 si PB5 ca iesire. */
	DDRB |= (1 << PB3);
	DDRB |= (1 << PB4);
	DDRB |= (1 << PB5);

	/* Sting LED-ul verde si ledul albastru. */
	PORTB &= ~(1 << PB4);
	PORTB &= ~(1 << PB5);
	
	/*Aprind LED-ul rosu*/
	PORTB |= (1 << PB3);
	
	/* Setez octava initiala*/
	octava = 1;
	
	/* Setez pinul PD5 ca iesire. */
	DDRD |= (1 << PD5);

	/* Opresc buzzer-ul. */
	PORTD &= ~(1 << PD5);
	
	/*Setez pinii PA0-PA7 ca intrare*/
	DDRA &= ~(1 << PA0);
	DDRA &= ~(1 << PA1);
	DDRA &= ~(1 << PA2);
	DDRA &= ~(1 << PA3);
	DDRA &= ~(1 << PA4);
	DDRA &= ~(1 << PA5);
	DDRA &= ~(1 << PA6);
	DDRA &= ~(1 << PA7);
	
	/* Activez rezistenta de pull-up pentru pinii PA0-PA7*/
	PORTA |= (1 << PA0) | (1 << PA1) | (1 << PA2) | (1 << PA3) |
				(1 << PA4) | (1 << PA5) | (1 << PA6) | (1 << PA7);
}

/* Genereaza o nota folosind buzzer-ul. */
void buzzer_reda(int nota) {
	float perioada_nota;
	int i, j;
	int durata_nota;

	if (octava == OCTAVA1) {
		perioada_nota = 100000 / frecvente_octava1[nota];
		durata_nota = DURATA_NOTA;
	}
	else if (octava == OCTAVA2) {
		perioada_nota = 100000 / frecvente_octava2[nota];
		durata_nota = DURATA_NOTA;
	}
	else {
		durata_nota = DURATA_NOTA_OCTAVA3;
		perioada_nota = 100000 / frecvente_octava3[nota];
	}

	
	for (i = 0; i < durata_nota; ++i) {
		PORTD |= (1 << PD5);

		/*
		 * Nu putem folosi variable ca argument pentru functia
		 * _delay_us(), asa ca folosim acest "workaround".
		 */
		for (j = 0; j < perioada_nota / 2; ++j)
			_delay_us(1);

		PORTD &= ~(1 << PD5);

		/*
		 * Nu putem folosi variable ca argument pentru functia
		 * _delay_us(), asa ca folosim acest "workaround".
		 */
		for (j = 0 ; j < perioada_nota / 2; ++j)
			_delay_us(1);
	}
}

void ADC_init(void)
{
	// enable ADC with:
	// *reference AVCC with external capacitor at AREF pin
	// *without left adjust of conversion result
	// *no auto-trigger
	// *no interrupt
	// *prescaler at 128
	ADMUX = (1 << REFS0);
	ADCSRA = (1 << ADEN) | (7 << ADPS0);
}

int ADC_get(uint8_t channel)
{
	// start ADC conversion on "channel"
	// wait for completion
	// return the result
	ADMUX = (ADMUX & ~(0x1f << MUX0)) | channel;

	ADCSRA |= (1 << ADSC);
	while(ADCSRA & (1 << ADSC));

	return ADC;
}

double ADC_voltage(int raw)
{
	// returnati conversia datelor ADC raw in volti
	return raw * AREF / 1024.;
}

void check_photoresistors(void)
{
	int i;
	double value;
	
	
	for (i = 0; i < NUM_NOTE; ++i) {
		/* Citeste tensiunea de pe canal */
		value = ADC_voltage(ADC_get(i));
		if (i != 3) {
			/* Daca tensiunea se afla deasupra pragului, reda nota */
			if (value > THRESHOLD)
				buzzer_reda(i);
			/* Prag special pentru canalul PA3, deoarece laserul era
			 * dispersat la momentul implementarii */
		} else {
			if (value > 1.2)
				buzzer_reda(i);
		}
	}
}

void change_octave()
{
	/* Daca butonul conectat la PB0 este apasat. */
	if ((PINB & (1 << PB0)) == 0) {
		/* Aprind LED-ul corespunzator */
		PORTB |= (1 << PB5);
		
		/* Sting celelalte doua LED-uri */
		PORTB &= ~(1 << PB3);
		PORTB &= ~(1 << PB4);
		
		/* Schimb octava */
		octava = 3;
	}
	/* Daca butonul conectat la PB1 este apasat. */
	else if ((PINB & (1 << PB1)) == 0) {
		/* Aprind LED-ul corespunzator */
		PORTB |= (1 << PB4);
		
		/* Sting celelalte doua LED-uri */
		PORTB &= ~(1 << PB3);
		PORTB &= ~(1 << PB5);
		
		/* Schimb octava */
		octava = 2;
	}
	/* Daca butonul conectat la PB2 este apasat. */
	else if ((PINB & (1 << PB2)) == 0) {
		/* Aprind LED-ul corespunzator */
		PORTB |= (1 << PB3);
		
		/* Sting celelalte LED-uri */
		PORTB &= ~(1 << PB4);
		PORTB &= ~(1 << PB5);
		
		/* Schimb octava */
		octava = 1;
	}

}


/* Reproduce nota muzicala corespunzatoare laserului interupt */
void play(void) 
{
	while (1) {
		/* Verifica ce note trebuie reproduse. */
		check_photoresistors();
		
		/* Schimba octava daca este nevoie */
		change_octave();
		
		_delay_ms(200);
	}
}


int main(void) {
	IO_init();
	ADC_init();
	play();

	return 0;
}
